@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, print_function
-
 import numpy as np
 import pandas as pd
 
@@ -15,8 +13,8 @@ window = 20
 
 randn_xy = randn((nx, ny), frac_nan=0.1)
 randn_xt = randn((nx, nt))
-randn_t = randn((nt, ))
-randn_long = randn((long_nx, ), frac_nan=0.1)
+randn_t = randn((nt,))
+randn_long = randn((long_nx,), frac_nan=0.1)
 
 
 new_x_short = np.linspace(0.3 * nx, 0.7 * nx, 100)
@@ -27,22 +25,25 @@ new_y_long = np.linspace(0.1, 0.9, 1000)
 class Interpolation:
     def setup(self, *args, **kwargs):
         self.ds = xr.Dataset(
-            {'var1': (('x', 'y'), randn_xy),
-             'var2': (('x', 't'), randn_xt),
-             'var3': (('t', ), randn_t)},
-            coords={'x': np.arange(nx),
-                    'y': np.linspace(0, 1, ny),
-                    't': pd.date_range('1970-01-01', periods=nt, freq='D'),
-                    'x_coords': ('x', np.linspace(1.1, 2.1, nx))})
+            {
+                "var1": (("x", "y"), randn_xy),
+                "var2": (("x", "t"), randn_xt),
+                "var3": (("t",), randn_t),
+            },
+            coords={
+                "x": np.arange(nx),
+                "y": np.linspace(0, 1, ny),
+                "t": pd.date_range("1970-01-01", periods=nt, freq="D"),
+                "x_coords": ("x", np.linspace(1.1, 2.1, nx)),
+            },
+        )
 
-    @parameterized(['method', 'is_short'],
-                   (['linear', 'cubic'], [True, False]))
+    @parameterized(["method", "is_short"], (["linear", "cubic"], [True, False]))
     def time_interpolation(self, method, is_short):
         new_x = new_x_short if is_short else new_x_long
         self.ds.interp(x=new_x, method=method).load()
 
-    @parameterized(['method'],
-                   (['linear', 'nearest']))
+    @parameterized(["method"], (["linear", "nearest"]))
     def time_interpolation_2d(self, method):
         self.ds.interp(x=new_x_long, y=new_y_long, method=method).load()
 
@@ -51,4 +52,4 @@ class InterpolationDask(Interpolation):
     def setup(self, *args, **kwargs):
         requires_dask()
         super().setup(**kwargs)
-        self.ds = self.ds.chunk({'t': 50})
+        self.ds = self.ds.chunk({"t": 50})
