@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 .. _recipes.rasterio:
 
@@ -24,27 +23,32 @@ from rasterio.warp import transform
 import xarray as xr
 
 # Read the data
-url = 'https://github.com/mapbox/rasterio/raw/master/tests/data/RGB.byte.tif'
+url = "https://github.com/mapbox/rasterio/raw/master/tests/data/RGB.byte.tif"
 da = xr.open_rasterio(url)
 
 # Compute the lon/lat coordinates with rasterio.warp.transform
-ny, nx = len(da['y']), len(da['x'])
-x, y = np.meshgrid(da['x'], da['y'])
+ny, nx = len(da["y"]), len(da["x"])
+x, y = np.meshgrid(da["x"], da["y"])
 
 # Rasterio works with 1D arrays
-lon, lat = transform(da.crs, {'init': 'EPSG:4326'},
-                     x.flatten(), y.flatten())
+lon, lat = transform(da.crs, {"init": "EPSG:4326"}, x.flatten(), y.flatten())
 lon = np.asarray(lon).reshape((ny, nx))
 lat = np.asarray(lat).reshape((ny, nx))
-da.coords['lon'] = (('y', 'x'), lon)
-da.coords['lat'] = (('y', 'x'), lat)
+da.coords["lon"] = (("y", "x"), lon)
+da.coords["lat"] = (("y", "x"), lat)
 
 # Compute a greyscale out of the rgb image
-greyscale = da.mean(dim='band')
+greyscale = da.mean(dim="band")
 
 # Plot on a map
 ax = plt.subplot(projection=ccrs.PlateCarree())
-greyscale.plot(ax=ax, x='lon', y='lat', transform=ccrs.PlateCarree(),
-               cmap='Greys_r', add_colorbar=False)
-ax.coastlines('10m', color='r')
+greyscale.plot(
+    ax=ax,
+    x="lon",
+    y="lat",
+    transform=ccrs.PlateCarree(),
+    cmap="Greys_r",
+    add_colorbar=False,
+)
+ax.coastlines("10m", color="r")
 plt.show()
