@@ -214,9 +214,10 @@ class DatasetCoordinates(Coordinates):
         return cast("DataArray", self._data[key])
 
     def to_dataset(self) -> "Dataset":
-        """Convert these coordinates into a new Dataset
-        """
-        return self._data._copy_listed(self._names)
+        """Convert these coordinates into a new Dataset"""
+
+        names = [name for name in self._data._variables if name in self._names]
+        return self._data._copy_listed(names)
 
     def _update_coords(
         self, coords: Dict[Hashable, Variable], indexes: Mapping[Hashable, pd.Index]
@@ -247,7 +248,7 @@ class DatasetCoordinates(Coordinates):
         if key in self:
             del self._data[key]
         else:
-            raise KeyError(key)
+            raise KeyError(f"{key!r} is not a coordinate variable.")
 
     def _ipython_key_completions_(self):
         """Provide method for the key-autocompletions in IPython. """
@@ -291,7 +292,7 @@ class DataArrayCoordinates(Coordinates):
         dims = calculate_dimensions(coords_plus_data)
         if not set(dims) <= set(self.dims):
             raise ValueError(
-                "cannot add coordinates with new dimensions to " "a DataArray"
+                "cannot add coordinates with new dimensions to a DataArray"
             )
         self._data._coords = coords
 
@@ -312,7 +313,12 @@ class DataArrayCoordinates(Coordinates):
         return Dataset._construct_direct(coords, set(coords))
 
     def __delitem__(self, key: Hashable) -> None:
-        del self._data._coords[key]
+        if key in self:
+            del self._data._coords[key]
+            if self._data._indexes is not None and key in self._data._indexes:
+                del self._data._indexes[key]
+        else:
+            raise KeyError(f"{key!r} is not a coordinate variable.")
 
     def _ipython_key_completions_(self):
         """Provide method for the key-autocompletions in IPython. """
