@@ -2,8 +2,9 @@
 Use this module directly:
     import xarray.plot as xplt
 
-Or use the methods on a DataArray:
+Or use the methods on a DataArray or Dataset:
     DataArray.plot._____
+    Dataset.plot._____
 """
 import functools
 
@@ -12,32 +13,39 @@ import pandas as pd
 
 from .facetgrid import _easy_facetgrid
 from .utils import (
-    _add_colorbar, _ensure_plottable, _infer_interval_breaks, _infer_xy_labels,
-    _interval_to_double_bound_points, _interval_to_mid_points,
-    _process_cmap_cbar_kwargs, _rescale_imshow_rgb, _resolve_intervals_2dplot,
-    _update_axes, _valid_other_type, get_axis, import_matplotlib_pyplot,
-    label_from_attrs)
+    _add_colorbar,
+    _assert_valid_xy,
+    _ensure_plottable,
+    _infer_interval_breaks,
+    _infer_xy_labels,
+    _process_cmap_cbar_kwargs,
+    _rescale_imshow_rgb,
+    _resolve_intervals_1dplot,
+    _resolve_intervals_2dplot,
+    _update_axes,
+    get_axis,
+    import_matplotlib_pyplot,
+    label_from_attrs,
+)
 
 
 def _infer_line_data(darray, x, y, hue):
-    error_msg = ('must be either None or one of ({0:s})'
-                 .format(', '.join([repr(dd) for dd in darray.dims])))
+
     ndims = len(darray.dims)
 
-    if x is not None and x not in darray.dims and x not in darray.coords:
-        raise ValueError('x ' + error_msg)
-
-    if y is not None and y not in darray.dims and y not in darray.coords:
-        raise ValueError('y ' + error_msg)
-
     if x is not None and y is not None:
-        raise ValueError('You cannot specify both x and y kwargs'
-                         'for line plots.')
+        raise ValueError("Cannot specify both x and y kwargs for line plots.")
+
+    if x is not None:
+        _assert_valid_xy(darray, x, "x")
+
+    if y is not None:
+        _assert_valid_xy(darray, y, "y")
 
     if ndims == 1:
         huename = None
         hueplt = None
-        huelabel = ''
+        huelabel = ""
 
         if x is not None:
             xplt = darray[x]
@@ -54,8 +62,7 @@ def _infer_line_data(darray, x, y, hue):
 
     else:
         if x is None and y is None and hue is None:
-            raise ValueError('For 2D inputs, please'
-                             'specify either hue, x or y.')
+            raise ValueError("For 2D inputs, please specify either hue, x or y.")
 
         if y is None:
             xname, huename = _infer_xy_labels(darray=darray, x=x, y=hue)
@@ -64,14 +71,18 @@ def _infer_line_data(darray, x, y, hue):
                 if huename in darray.dims:
                     otherindex = 1 if darray.dims.index(huename) == 0 else 0
                     otherdim = darray.dims[otherindex]
-                    yplt = darray.transpose(otherdim, huename)
-                    xplt = xplt.transpose(otherdim, huename)
+                    yplt = darray.transpose(otherdim, huename, transpose_coords=False)
+                    xplt = xplt.transpose(otherdim, huename, transpose_coords=False)
                 else:
-                    raise ValueError('For 2D inputs, hue must be a dimension'
-                                     + ' i.e. one of ' + repr(darray.dims))
+                    raise ValueError(
+                        "For 2D inputs, hue must be a dimension"
+                        " i.e. one of " + repr(darray.dims)
+                    )
 
             else:
-                yplt = darray.transpose(xname, huename)
+                (xdim,) = darray[xname].dims
+                (huedim,) = darray[huename].dims
+                yplt = darray.transpose(xdim, huedim)
 
         else:
             yname, huename = _infer_xy_labels(darray=darray, x=y, y=hue)
@@ -79,13 +90,19 @@ def _infer_line_data(darray, x, y, hue):
             if yplt.ndim > 1:
                 if huename in darray.dims:
                     otherindex = 1 if darray.dims.index(huename) == 0 else 0
-                    xplt = darray.transpose(otherdim, huename)
+                    otherdim = darray.dims[otherindex]
+                    xplt = darray.transpose(otherdim, huename, transpose_coords=False)
+                    yplt = yplt.transpose(otherdim, huename, transpose_coords=False)
                 else:
-                    raise ValueError('For 2D inputs, hue must be a dimension'
-                                     + ' i.e. one of ' + repr(darray.dims))
+                    raise ValueError(
+                        "For 2D inputs, hue must be a dimension"
+                        " i.e. one of " + repr(darray.dims)
+                    )
 
             else:
-                xplt = darray.transpose(yname, huename)
+                (ydim,) = darray[yname].dims
+                (huedim,) = darray[huename].dims
+                xplt = darray.transpose(ydim, huedim)
 
         huelabel = label_from_attrs(darray[huename])
         hueplt = darray[huename]
@@ -96,8 +113,17 @@ def _infer_line_data(darray, x, y, hue):
     return xplt, yplt, hueplt, xlabel, ylabel, huelabel
 
 
-def plot(darray, row=None, col=None, col_wrap=None, ax=None, hue=None,
-         rtol=0.01, subplot_kws=None, **kwargs):
+def plot(
+    darray,
+    row=None,
+    col=None,
+    col_wrap=None,
+    ax=None,
+    hue=None,
+    rtol=0.01,
+    subplot_kws=None,
+    **kwargs,
+):
     """
     Default plot of DataArray using matplotlib.pyplot.
 
@@ -115,27 +141,26 @@ def plot(darray, row=None, col=None, col_wrap=None, ax=None, hue=None,
     Parameters
     ----------
     darray : DataArray
-    row : string, optional
+    row : str, optional
         If passed, make row faceted plots on this dimension name
-    col : string, optional
+    col : str, optional
         If passed, make column faceted plots on this dimension name
-    hue : string, optional
+    hue : str, optional
         If passed, make faceted line plots with hue on this dimension name
-    col_wrap : integer, optional
+    col_wrap : int, optional
         Use together with ``col`` to wrap faceted plots
-    ax : matplotlib axes, optional
+    ax : matplotlib.axes.Axes, optional
         If None, uses the current axis. Not applicable when using facets.
-    rtol : number, optional
+    rtol : float, optional
         Relative tolerance used to determine if the indexes
         are uniformly spaced. Usually a small positive number.
     subplot_kws : dict, optional
-        Dictionary of keyword arguments for matplotlib subplots. Only applies
-        to FacetGrid plotting.
+        Dictionary of keyword arguments for matplotlib subplots.
     **kwargs : optional
         Additional keyword arguments to matplotlib
 
     """
-    darray = darray.squeeze()
+    darray = darray.squeeze().compute()
 
     plot_dims = set(darray.dims)
     plot_dims.discard(row)
@@ -144,37 +169,63 @@ def plot(darray, row=None, col=None, col_wrap=None, ax=None, hue=None,
 
     ndims = len(plot_dims)
 
-    error_msg = ('Only 1d and 2d plots are supported for facets in xarray. '
-                 'See the package `Seaborn` for more options.')
+    error_msg = (
+        "Only 1d and 2d plots are supported for facets in xarray. "
+        "See the package `Seaborn` for more options."
+    )
 
     if ndims in [1, 2]:
         if row or col:
-            kwargs['row'] = row
-            kwargs['col'] = col
-            kwargs['col_wrap'] = col_wrap
-            kwargs['subplot_kws'] = subplot_kws
+            kwargs["subplot_kws"] = subplot_kws
+            kwargs["row"] = row
+            kwargs["col"] = col
+            kwargs["col_wrap"] = col_wrap
         if ndims == 1:
             plotfunc = line
-            kwargs['hue'] = hue
+            kwargs["hue"] = hue
         elif ndims == 2:
             if hue:
                 plotfunc = line
-                kwargs['hue'] = hue
+                kwargs["hue"] = hue
             else:
                 plotfunc = pcolormesh
+                kwargs["subplot_kws"] = subplot_kws
     else:
         if row or col or hue:
             raise ValueError(error_msg)
         plotfunc = hist
 
-    kwargs['ax'] = ax
+    kwargs["ax"] = ax
 
     return plotfunc(darray, **kwargs)
 
 
 # This function signature should not change so that it can use
 # matplotlib format strings
-def line(darray, *args, **kwargs):
+def line(
+    darray,
+    *args,
+    row=None,
+    col=None,
+    figsize=None,
+    aspect=None,
+    size=None,
+    ax=None,
+    hue=None,
+    x=None,
+    y=None,
+    xincrease=None,
+    yincrease=None,
+    xscale=None,
+    yscale=None,
+    xticks=None,
+    yticks=None,
+    xlim=None,
+    ylim=None,
+    add_legend=True,
+    _labels=True,
+    **kwargs,
+):
     """
     Line plot of DataArray index against values
 
@@ -200,7 +251,7 @@ def line(darray, *args, **kwargs):
         Dimension or coordinate for which you want multiple lines plotted.
         If plotting against a 2D coordinate, ``hue`` must be a dimension.
     x, y : string, optional
-        Dimensions or coordinates for x, y axis.
+        Dimension, coordinate or MultiIndex level for x, y axis.
         Only one of these may be specified.
         The other coordinate plots values from the DataArray on which this
         plot method is called.
@@ -214,73 +265,39 @@ def line(darray, *args, **kwargs):
     yincrease : None, True, or False, optional
         Should the values on the y axes be increasing from top to bottom?
         if None, use the default for the matplotlib function.
-    add_legend : boolean, optional
+    add_legend : bool, optional
         Add legend with y axis coordinates (2D inputs only).
     *args, **kwargs : optional
         Additional arguments to matplotlib.pyplot.plot
-
     """
-
     # Handle facetgrids first
-    row = kwargs.pop('row', None)
-    col = kwargs.pop('col', None)
     if row or col:
         allargs = locals().copy()
-        allargs.update(allargs.pop('kwargs'))
-        allargs.pop('darray')
-        return _easy_facetgrid(darray, line, kind='line', **allargs)
+        allargs.update(allargs.pop("kwargs"))
+        allargs.pop("darray")
+        return _easy_facetgrid(darray, line, kind="line", **allargs)
 
     ndims = len(darray.dims)
     if ndims > 2:
-        raise ValueError('Line plots are for 1- or 2-dimensional DataArrays. '
-                         'Passed DataArray has {ndims} '
-                         'dimensions'.format(ndims=ndims))
+        raise ValueError(
+            "Line plots are for 1- or 2-dimensional DataArrays. "
+            "Passed DataArray has {ndims} "
+            "dimensions".format(ndims=ndims)
+        )
 
-    # Ensures consistency with .plot method
-    figsize = kwargs.pop('figsize', None)
-    aspect = kwargs.pop('aspect', None)
-    size = kwargs.pop('size', None)
-    ax = kwargs.pop('ax', None)
-    hue = kwargs.pop('hue', None)
-    x = kwargs.pop('x', None)
-    y = kwargs.pop('y', None)
-    xincrease = kwargs.pop('xincrease', None)  # default needs to be None
-    yincrease = kwargs.pop('yincrease', None)
-    xscale = kwargs.pop('xscale', None)  # default needs to be None
-    yscale = kwargs.pop('yscale', None)
-    xticks = kwargs.pop('xticks', None)
-    yticks = kwargs.pop('yticks', None)
-    xlim = kwargs.pop('xlim', None)
-    ylim = kwargs.pop('ylim', None)
-    add_legend = kwargs.pop('add_legend', True)
-    _labels = kwargs.pop('_labels', True)
-    if args is ():
-        args = kwargs.pop('args', ())
+    # The allargs dict passed to _easy_facetgrid above contains args
+    if args == ():
+        args = kwargs.pop("args", ())
+    else:
+        assert "args" not in kwargs
 
     ax = get_axis(figsize, size, aspect, ax)
-    xplt, yplt, hueplt, xlabel, ylabel, huelabel = \
-        _infer_line_data(darray, x, y, hue)
+    xplt, yplt, hueplt, xlabel, ylabel, hue_label = _infer_line_data(darray, x, y, hue)
 
-    # Remove pd.Intervals if contained in xplt.values.
-    if _valid_other_type(xplt.values, [pd.Interval]):
-        # Is it a step plot? (see matplotlib.Axes.step)
-        if kwargs.get('linestyle', '').startswith('steps-'):
-            xplt_val, yplt_val = _interval_to_double_bound_points(xplt.values,
-                                                                  yplt.values)
-            # Remove steps-* to be sure that matplotlib is not confused
-            kwargs['linestyle'] = (kwargs['linestyle']
-                                   .replace('steps-pre', '')
-                                   .replace('steps-post', '')
-                                   .replace('steps-mid', ''))
-            if kwargs['linestyle'] == '':
-                kwargs.pop('linestyle')
-        else:
-            xplt_val = _interval_to_mid_points(xplt.values)
-            yplt_val = yplt.values
-            xlabel += '_center'
-    else:
-        xplt_val = xplt.values
-        yplt_val = yplt.values
+    # Remove pd.Intervals if contained in xplt.values and/or yplt.values.
+    xplt_val, yplt_val, xlabel, ylabel, kwargs = _resolve_intervals_1dplot(
+        xplt.values, yplt.values, xlabel, ylabel, kwargs
+    )
 
     _ensure_plottable(xplt_val, yplt_val)
 
@@ -296,9 +313,7 @@ def line(darray, *args, **kwargs):
         ax.set_title(darray._title_for_slice())
 
     if darray.ndim == 2 and add_legend:
-        ax.legend(handles=primitive,
-                  labels=list(hueplt.values),
-                  title=huelabel)
+        ax.legend(handles=primitive, labels=list(hueplt.values), title=hue_label)
 
     # Rotate dates on xlabels
     # Do this without calling autofmt_xdate so that x-axes ticks
@@ -307,15 +322,14 @@ def line(darray, *args, **kwargs):
     if np.issubdtype(xplt.dtype, np.datetime64):
         for xlabels in ax.get_xticklabels():
             xlabels.set_rotation(30)
-            xlabels.set_ha('right')
+            xlabels.set_ha("right")
 
-    _update_axes(ax, xincrease, yincrease, xscale, yscale,
-                 xticks, yticks, xlim, ylim)
+    _update_axes(ax, xincrease, yincrease, xscale, yscale, xticks, yticks, xlim, ylim)
 
     return primitive
 
 
-def step(darray, *args, **kwargs):
+def step(darray, *args, where="pre", drawstyle=None, ds=None, **kwargs):
     """
     Step plot of DataArray index against values
 
@@ -323,39 +337,56 @@ def step(darray, *args, **kwargs):
 
     Parameters
     ----------
-    where : {'pre', 'post', 'mid'}, optional, default 'pre'
+    where : {"pre", "post", "mid"}, default: "pre"
         Define where the steps should be placed:
-        - 'pre': The y value is continued constantly to the left from
+
+        - "pre": The y value is continued constantly to the left from
           every *x* position, i.e. the interval ``(x[i-1], x[i]]`` has the
           value ``y[i]``.
-        - 'post': The y value is continued constantly to the right from
+        - "post": The y value is continued constantly to the right from
           every *x* position, i.e. the interval ``[x[i], x[i+1])`` has the
           value ``y[i]``.
-        - 'mid': Steps occur half-way between the *x* positions.
-        Note that this parameter is ignored if the x coordinate consists of
+        - "mid": Steps occur half-way between the *x* positions.
+
+        Note that this parameter is ignored if one coordinate consists of
         :py:func:`pandas.Interval` values, e.g. as a result of
         :py:func:`xarray.Dataset.groupby_bins`. In this case, the actual
         boundaries of the interval are used.
 
     *args, **kwargs : optional
         Additional arguments following :py:func:`xarray.plot.line`
-
     """
-    if ('ls' in kwargs.keys()) and ('linestyle' not in kwargs.keys()):
-        kwargs['linestyle'] = kwargs.pop('ls')
+    if where not in {"pre", "post", "mid"}:
+        raise ValueError("'where' argument to step must be 'pre', 'post' or 'mid'")
 
-    where = kwargs.pop('where', 'pre')
+    if ds is not None:
+        if drawstyle is None:
+            drawstyle = ds
+        else:
+            raise TypeError("ds and drawstyle are mutually exclusive")
+    if drawstyle is None:
+        drawstyle = ""
+    drawstyle = "steps-" + where + drawstyle
 
-    if where not in ('pre', 'post', 'mid'):
-        raise ValueError("'where' argument to step must be "
-                         "'pre', 'post' or 'mid'")
-
-    kwargs['linestyle'] = 'steps-' + where + kwargs.get('linestyle', '')
-
-    return line(darray, *args, **kwargs)
+    return line(darray, *args, drawstyle=drawstyle, **kwargs)
 
 
-def hist(darray, figsize=None, size=None, aspect=None, ax=None, **kwargs):
+def hist(
+    darray,
+    figsize=None,
+    size=None,
+    aspect=None,
+    ax=None,
+    xincrease=None,
+    yincrease=None,
+    xscale=None,
+    yscale=None,
+    xticks=None,
+    yticks=None,
+    xlim=None,
+    ylim=None,
+    **kwargs,
+):
     """
     Histogram of DataArray
 
@@ -376,7 +407,7 @@ def hist(darray, figsize=None, size=None, aspect=None, ax=None, **kwargs):
     size : scalar, optional
         If provided, create a new figure for the plot with the given size.
         Height (in inches) of each plot. See also: ``aspect``.
-    ax : matplotlib axes object, optional
+    ax : matplotlib.axes.Axes, optional
         Axis on which to plot this figure. By default, use the current axis.
         Mutually exclusive with ``size`` and ``figsize``.
     **kwargs : optional
@@ -385,43 +416,40 @@ def hist(darray, figsize=None, size=None, aspect=None, ax=None, **kwargs):
     """
     ax = get_axis(figsize, size, aspect, ax)
 
-    xincrease = kwargs.pop('xincrease', None)  # default needs to be None
-    yincrease = kwargs.pop('yincrease', None)
-    xscale = kwargs.pop('xscale', None)  # default needs to be None
-    yscale = kwargs.pop('yscale', None)
-    xticks = kwargs.pop('xticks', None)
-    yticks = kwargs.pop('yticks', None)
-    xlim = kwargs.pop('xlim', None)
-    ylim = kwargs.pop('ylim', None)
-
     no_nan = np.ravel(darray.values)
     no_nan = no_nan[pd.notnull(no_nan)]
 
     primitive = ax.hist(no_nan, **kwargs)
 
-    ax.set_title('Histogram')
+    ax.set_title("Histogram")
     ax.set_xlabel(label_from_attrs(darray))
 
-    _update_axes(ax, xincrease, yincrease, xscale, yscale,
-                 xticks, yticks, xlim, ylim)
+    _update_axes(ax, xincrease, yincrease, xscale, yscale, xticks, yticks, xlim, ylim)
 
     return primitive
 
 
 # MUST run before any 2d plotting functions are defined since
 # _plot2d decorator adds them as methods here.
-class _PlotMethods(object):
+class _PlotMethods:
     """
     Enables use of xarray.plot functions as attributes on a DataArray.
     For example, DataArray.plot.imshow
     """
 
+    __slots__ = ("_da",)
+
     def __init__(self, darray):
         self._da = darray
 
     def __call__(self, **kwargs):
         return plot(self._da, **kwargs)
 
+    # we can't use functools.wraps here since that also modifies the name / qualname
+    __doc__ = __call__.__doc__ = plot.__doc__
+    __call__.__wrapped__ = plot  # type: ignore
+    __call__.__annotations__ = plot.__annotations__
+
     @functools.wraps(hist)
     def hist(self, ax=None, **kwargs):
         return hist(self._da, ax=ax, **kwargs)
@@ -435,6 +463,15 @@ class _PlotMethods(object):
         return step(self._da, *args, **kwargs)
 
 
+def override_signature(f):
+    def wrapper(func):
+        func.__wrapped__ = f
+
+        return func
+
+    return wrapper
+
+
 def _plot2d(plotfunc):
     """
     Decorator for common 2d plotting logic
@@ -466,7 +503,7 @@ def _plot2d(plotfunc):
         If passed, make row faceted plots on this dimension name
     col : string, optional
         If passed, make column faceted plots on this dimension name
-    col_wrap : integer, optional
+    col_wrap : int, optional
         Use together with ``col`` to wrap faceted plots
     xscale, yscale : 'linear', 'symlog', 'log', 'logit', optional
         Specifies scaling for the x- and y-axes respectively
@@ -478,9 +515,9 @@ def _plot2d(plotfunc):
     yincrease : None, True, or False, optional
         Should the values on the y axes be increasing from top to bottom?
         if None, use the default for the matplotlib function.
-    add_colorbar : Boolean, optional
+    add_colorbar : bool, optional
         Adds colorbar to axis
-    add_labels : Boolean, optional
+    add_labels : bool, optional
         Use xarray metadata to label axes
     norm : ``matplotlib.colors.Normalize`` instance, optional
         If the ``norm`` has vmin or vmax specified, the corresponding kwarg
@@ -509,7 +546,7 @@ def _plot2d(plotfunc):
     robust : bool, optional
         If True and ``vmin`` or ``vmax`` are absent, the colormap range is
         computed with 2nd and 98th percentiles instead of the extreme values.
-    extend : {'neither', 'both', 'min', 'max'}, optional
+    extend : {"neither", "both", "min", "max"}, optional
         How to draw arrows extending the colorbar beyond its limits. If not
         provided, extend is inferred from vmin, vmax and the data limits.
     levels : int or list-like object, optional
@@ -525,8 +562,8 @@ def _plot2d(plotfunc):
         always infer intervals, unless the mesh is irregular and plotted on
         a map projection.
     subplot_kws : dict, optional
-        Dictionary of keyword arguments for matplotlib subplots. Only applies
-        to FacetGrid plotting.
+        Dictionary of keyword arguments for matplotlib subplots. Only used
+        for 2D and FacetGrid plots.
     cbar_ax : matplotlib Axes, optional
         Axes in which to draw the colorbar.
     cbar_kwargs : dict, optional
@@ -542,27 +579,64 @@ def _plot2d(plotfunc):
     """
 
     # Build on the original docstring
-    plotfunc.__doc__ = '%s\n%s' % (plotfunc.__doc__, commondoc)
+    plotfunc.__doc__ = f"{plotfunc.__doc__}\n{commondoc}"
 
+    # plotfunc and newplotfunc have different signatures:
+    # - plotfunc: (x, y, z, ax, **kwargs)
+    # - newplotfunc: (darray, x, y, **kwargs)
+    # where plotfunc accepts numpy arrays, while newplotfunc accepts a DataArray
+    # and variable names. newplotfunc also explicitly lists most kwargs, so we
+    # need to shorten it
+    def signature(darray, x, y, **kwargs):
+        pass
+
+    @override_signature(signature)
     @functools.wraps(plotfunc)
-    def newplotfunc(darray, x=None, y=None, figsize=None, size=None,
-                    aspect=None, ax=None, row=None, col=None,
-                    col_wrap=None, xincrease=True, yincrease=True,
-                    add_colorbar=None, add_labels=True, vmin=None, vmax=None,
-                    cmap=None, center=None, robust=False, extend=None,
-                    levels=None, infer_intervals=None, colors=None,
-                    subplot_kws=None, cbar_ax=None, cbar_kwargs=None,
-                    xscale=None, yscale=None, xticks=None, yticks=None,
-                    xlim=None, ylim=None, norm=None, **kwargs):
+    def newplotfunc(
+        darray,
+        x=None,
+        y=None,
+        figsize=None,
+        size=None,
+        aspect=None,
+        ax=None,
+        row=None,
+        col=None,
+        col_wrap=None,
+        xincrease=True,
+        yincrease=True,
+        add_colorbar=None,
+        add_labels=True,
+        vmin=None,
+        vmax=None,
+        cmap=None,
+        center=None,
+        robust=False,
+        extend=None,
+        levels=None,
+        infer_intervals=None,
+        colors=None,
+        subplot_kws=None,
+        cbar_ax=None,
+        cbar_kwargs=None,
+        xscale=None,
+        yscale=None,
+        xticks=None,
+        yticks=None,
+        xlim=None,
+        ylim=None,
+        norm=None,
+        **kwargs,
+    ):
         # All 2d plots in xarray share this function signature.
         # Method signature below should be consistent.
 
         # Decide on a default for the colorbar before facetgrids
         if add_colorbar is None:
-            add_colorbar = plotfunc.__name__ != 'contour'
-        imshow_rgb = (
-            plotfunc.__name__ == 'imshow' and
-            darray.ndim == (3 + (row is not None) + (col is not None)))
+            add_colorbar = plotfunc.__name__ != "contour"
+        imshow_rgb = plotfunc.__name__ == "imshow" and darray.ndim == (
+            3 + (row is not None) + (col is not None)
+        )
         if imshow_rgb:
             # Don't add a colorbar when showing an image with explicit colors
             add_colorbar = False
@@ -575,24 +649,27 @@ def _plot2d(plotfunc):
         # Handle facetgrids first
         if row or col:
             allargs = locals().copy()
-            allargs.pop('imshow_rgb')
-            allargs.update(allargs.pop('kwargs'))
-            allargs.pop('darray')
+            del allargs["darray"]
+            del allargs["imshow_rgb"]
+            allargs.update(allargs.pop("kwargs"))
             # Need the decorated plotting function
-            allargs['plotfunc'] = globals()[plotfunc.__name__]
-            return _easy_facetgrid(darray, kind='dataarray', **allargs)
+            allargs["plotfunc"] = globals()[plotfunc.__name__]
+            return _easy_facetgrid(darray, kind="dataarray", **allargs)
 
         plt = import_matplotlib_pyplot()
 
-        rgb = kwargs.pop('rgb', None)
-        if rgb is not None and plotfunc.__name__ != 'imshow':
+        rgb = kwargs.pop("rgb", None)
+        if rgb is not None and plotfunc.__name__ != "imshow":
             raise ValueError('The "rgb" keyword is only valid for imshow()')
         elif rgb is not None and not imshow_rgb:
-            raise ValueError('The "rgb" keyword is only valid for imshow()'
-                             'with a three-dimensional array (per facet)')
+            raise ValueError(
+                'The "rgb" keyword is only valid for imshow()'
+                "with a three-dimensional array (per facet)"
+            )
 
         xlab, ylab = _infer_xy_labels(
-            darray=darray, x=x, y=y, imshow=imshow_rgb, rgb=rgb)
+            darray=darray, x=x, y=y, imshow=imshow_rgb, rgb=rgb
+        )
 
         # better to pass the ndarrays directly to plotting functions
         xval = darray[xlab].values
@@ -600,10 +677,22 @@ def _plot2d(plotfunc):
 
         # check if we need to broadcast one dimension
         if xval.ndim < yval.ndim:
-            xval = np.broadcast_to(xval, yval.shape)
+            dims = darray[ylab].dims
+            if xval.shape[0] == yval.shape[0]:
+                xval = np.broadcast_to(xval[:, np.newaxis], yval.shape)
+            else:
+                xval = np.broadcast_to(xval[np.newaxis, :], yval.shape)
 
-        if yval.ndim < xval.ndim:
-            yval = np.broadcast_to(yval, xval.shape)
+        elif yval.ndim < xval.ndim:
+            dims = darray[xlab].dims
+            if yval.shape[0] == xval.shape[0]:
+                yval = np.broadcast_to(yval[:, np.newaxis], xval.shape)
+            else:
+                yval = np.broadcast_to(yval[np.newaxis, :], xval.shape)
+        elif xval.ndim == 2:
+            dims = darray[xlab].dims
+        else:
+            dims = (darray[ylab].dims[0], darray[xlab].dims[0])
 
         # May need to transpose for correct x, y labels
         # xlab may be the name of a coord, we have to check for dim names
@@ -613,10 +702,9 @@ def _plot2d(plotfunc):
             # we transpose to (y, x, color) to make this work.
             yx_dims = (ylab, xlab)
             dims = yx_dims + tuple(d for d in darray.dims if d not in yx_dims)
-            if dims != darray.dims:
-                darray = darray.transpose(*dims)
-        elif darray[xlab].dims[-1] == darray.dims[0]:
-            darray = darray.transpose()
+
+        if dims != darray.dims:
+            darray = darray.transpose(*dims, transpose_coords=True)
 
         # Pass the data as a masked ndarray too
         zval = darray.to_masked_array(copy=False)
@@ -625,37 +713,49 @@ def _plot2d(plotfunc):
         xplt, xlab_extra = _resolve_intervals_2dplot(xval, plotfunc.__name__)
         yplt, ylab_extra = _resolve_intervals_2dplot(yval, plotfunc.__name__)
 
-        _ensure_plottable(xplt, yplt)
+        _ensure_plottable(xplt, yplt, zval)
 
         cmap_params, cbar_kwargs = _process_cmap_cbar_kwargs(
-            plotfunc, locals(), zval.data)
+            plotfunc,
+            zval.data,
+            **locals(),
+            _is_facetgrid=kwargs.pop("_is_facetgrid", False),
+        )
 
-        if 'contour' in plotfunc.__name__:
+        if "contour" in plotfunc.__name__:
             # extend is a keyword argument only for contour and contourf, but
             # passing it to the colorbar is sufficient for imshow and
             # pcolormesh
-            kwargs['extend'] = cmap_params['extend']
-            kwargs['levels'] = cmap_params['levels']
+            kwargs["extend"] = cmap_params["extend"]
+            kwargs["levels"] = cmap_params["levels"]
             # if colors == a single color, matplotlib draws dashed negative
             # contours. we lose this feature if we pass cmap and not colors
             if isinstance(colors, str):
-                cmap_params['cmap'] = None
-                kwargs['colors'] = colors
+                cmap_params["cmap"] = None
+                kwargs["colors"] = colors
 
-        if 'pcolormesh' == plotfunc.__name__:
-            kwargs['infer_intervals'] = infer_intervals
+        if "pcolormesh" == plotfunc.__name__:
+            kwargs["infer_intervals"] = infer_intervals
 
-        if 'imshow' == plotfunc.__name__ and isinstance(aspect, str):
+        if "imshow" == plotfunc.__name__ and isinstance(aspect, str):
             # forbid usage of mpl strings
-            raise ValueError("plt.imshow's `aspect` kwarg is not available "
-                             "in xarray")
+            raise ValueError("plt.imshow's `aspect` kwarg is not available in xarray")
 
-        ax = get_axis(figsize, size, aspect, ax)
-        primitive = plotfunc(xplt, yplt, zval, ax=ax, cmap=cmap_params['cmap'],
-                             vmin=cmap_params['vmin'],
-                             vmax=cmap_params['vmax'],
-                             norm=cmap_params['norm'],
-                             **kwargs)
+        if subplot_kws is None:
+            subplot_kws = dict()
+        ax = get_axis(figsize, size, aspect, ax, **subplot_kws)
+
+        primitive = plotfunc(
+            xplt,
+            yplt,
+            zval,
+            ax=ax,
+            cmap=cmap_params["cmap"],
+            vmin=cmap_params["vmin"],
+            vmax=cmap_params["vmax"],
+            norm=cmap_params["norm"],
+            **kwargs,
+        )
 
         # Label the plot with metadata
         if add_labels:
@@ -664,22 +764,22 @@ def _plot2d(plotfunc):
             ax.set_title(darray._title_for_slice())
 
         if add_colorbar:
-            if add_labels and 'label' not in cbar_kwargs:
-                cbar_kwargs['label'] = label_from_attrs(darray)
-            cbar = _add_colorbar(primitive, ax, cbar_ax, cbar_kwargs,
-                                 cmap_params)
-
-        elif (cbar_ax is not None or cbar_kwargs):
+            if add_labels and "label" not in cbar_kwargs:
+                cbar_kwargs["label"] = label_from_attrs(darray)
+            cbar = _add_colorbar(primitive, ax, cbar_ax, cbar_kwargs, cmap_params)
+        elif cbar_ax is not None or cbar_kwargs:
             # inform the user about keywords which aren't used
-            raise ValueError("cbar_ax and cbar_kwargs can't be used with "
-                             "add_colorbar=False.")
+            raise ValueError(
+                "cbar_ax and cbar_kwargs can't be used with add_colorbar=False."
+            )
 
         # origin kwarg overrides yincrease
-        if 'origin' in kwargs:
+        if "origin" in kwargs:
             yincrease = None
 
-        _update_axes(ax, xincrease, yincrease, xscale, yscale,
-                     xticks, yticks, xlim, ylim)
+        _update_axes(
+            ax, xincrease, yincrease, xscale, yscale, xticks, yticks, xlim, ylim
+        )
 
         # Rotate dates on xlabels
         # Do this without calling autofmt_xdate so that x-axes ticks
@@ -688,21 +788,48 @@ def _plot2d(plotfunc):
         if np.issubdtype(xplt.dtype, np.datetime64):
             for xlabels in ax.get_xticklabels():
                 xlabels.set_rotation(30)
-                xlabels.set_ha('right')
+                xlabels.set_ha("right")
 
         return primitive
 
     # For use as DataArray.plot.plotmethod
     @functools.wraps(newplotfunc)
-    def plotmethod(_PlotMethods_obj, x=None, y=None, figsize=None, size=None,
-                   aspect=None, ax=None, row=None, col=None, col_wrap=None,
-                   xincrease=True, yincrease=True, add_colorbar=None,
-                   add_labels=True, vmin=None, vmax=None, cmap=None,
-                   colors=None, center=None, robust=False, extend=None,
-                   levels=None, infer_intervals=None, subplot_kws=None,
-                   cbar_ax=None, cbar_kwargs=None,
-                   xscale=None, yscale=None, xticks=None, yticks=None,
-                   xlim=None, ylim=None, norm=None, **kwargs):
+    def plotmethod(
+        _PlotMethods_obj,
+        x=None,
+        y=None,
+        figsize=None,
+        size=None,
+        aspect=None,
+        ax=None,
+        row=None,
+        col=None,
+        col_wrap=None,
+        xincrease=True,
+        yincrease=True,
+        add_colorbar=None,
+        add_labels=True,
+        vmin=None,
+        vmax=None,
+        cmap=None,
+        colors=None,
+        center=None,
+        robust=False,
+        extend=None,
+        levels=None,
+        infer_intervals=None,
+        subplot_kws=None,
+        cbar_ax=None,
+        cbar_kwargs=None,
+        xscale=None,
+        yscale=None,
+        xticks=None,
+        yticks=None,
+        xlim=None,
+        ylim=None,
+        norm=None,
+        **kwargs,
+    ):
         """
         The method should have the same signature as the function.
 
@@ -710,9 +837,9 @@ def _plot2d(plotfunc):
         and passes all the other arguments straight through.
         """
         allargs = locals()
-        allargs['darray'] = _PlotMethods_obj._da
+        allargs["darray"] = _PlotMethods_obj._da
         allargs.update(kwargs)
-        for arg in ['_PlotMethods_obj', 'newplotfunc', 'kwargs']:
+        for arg in ["_PlotMethods_obj", "newplotfunc", "kwargs"]:
             del allargs[arg]
         return newplotfunc(**allargs)
 
@@ -748,36 +875,36 @@ def imshow(x, y, z, ax, **kwargs):
     """
 
     if x.ndim != 1 or y.ndim != 1:
-        raise ValueError('imshow requires 1D coordinates, try using '
-                         'pcolormesh or contour(f)')
+        raise ValueError(
+            "imshow requires 1D coordinates, try using pcolormesh or contour(f)"
+        )
 
     # Centering the pixels- Assumes uniform spacing
     try:
         xstep = (x[1] - x[0]) / 2.0
     except IndexError:
         # Arbitrary default value, similar to matplotlib behaviour
-        xstep = .1
+        xstep = 0.1
     try:
         ystep = (y[1] - y[0]) / 2.0
     except IndexError:
-        ystep = .1
+        ystep = 0.1
     left, right = x[0] - xstep, x[-1] + xstep
     bottom, top = y[-1] + ystep, y[0] - ystep
 
-    defaults = {'origin': 'upper',
-                'interpolation': 'nearest'}
+    defaults = {"origin": "upper", "interpolation": "nearest"}
 
-    if not hasattr(ax, 'projection'):
+    if not hasattr(ax, "projection"):
         # not for cartopy geoaxes
-        defaults['aspect'] = 'auto'
+        defaults["aspect"] = "auto"
 
     # Allow user to override these defaults
     defaults.update(kwargs)
 
-    if defaults['origin'] == 'upper':
-        defaults['extent'] = [left, right, bottom, top]
+    if defaults["origin"] == "upper":
+        defaults["extent"] = [left, right, bottom, top]
     else:
-        defaults['extent'] = [left, right, top, bottom]
+        defaults["extent"] = [left, right, top, bottom]
 
     if z.ndim == 3:
         # matplotlib imshow uses black for missing data, but Xarray makes
@@ -830,7 +957,7 @@ def pcolormesh(x, y, z, ax, infer_intervals=None, **kwargs):
     # decide on a default for infer_intervals (GH781)
     x = np.asarray(x)
     if infer_intervals is None:
-        if hasattr(ax, 'projection'):
+        if hasattr(ax, "projection"):
             if len(x.shape) == 1:
                 infer_intervals = True
             else:
@@ -838,9 +965,10 @@ def pcolormesh(x, y, z, ax, infer_intervals=None, **kwargs):
         else:
             infer_intervals = True
 
-    if (infer_intervals and
-            ((np.shape(x)[0] == np.shape(z)[1]) or
-             ((x.ndim > 1) and (np.shape(x)[1] == np.shape(z)[1])))):
+    if infer_intervals and (
+        (np.shape(x)[0] == np.shape(z)[1])
+        or ((x.ndim > 1) and (np.shape(x)[1] == np.shape(z)[1]))
+    ):
         if len(x.shape) == 1:
             x = _infer_interval_breaks(x, check_monotonic=True)
         else:
@@ -848,8 +976,7 @@ def pcolormesh(x, y, z, ax, infer_intervals=None, **kwargs):
             x = _infer_interval_breaks(x, axis=1)
             x = _infer_interval_breaks(x, axis=0)
 
-    if (infer_intervals and
-            (np.shape(y)[0] == np.shape(z)[0])):
+    if infer_intervals and (np.shape(y)[0] == np.shape(z)[0]):
         if len(y.shape) == 1:
             y = _infer_interval_breaks(y, check_monotonic=True)
         else:
@@ -861,7 +988,7 @@ def pcolormesh(x, y, z, ax, infer_intervals=None, **kwargs):
 
     # by default, pcolormesh picks "round" values for bounds
     # this results in ugly looking plots with lots of surrounding whitespace
-    if not hasattr(ax, 'projection') and x.ndim == 1 and y.ndim == 1:
+    if not hasattr(ax, "projection") and x.ndim == 1 and y.ndim == 1:
         # not a cartopy geoaxis
         ax.set_xlim(x[0], x[-1])
         ax.set_ylim(y[0], y[-1])
