@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 from contextlib import suppress
 
 import numpy as np
@@ -9,8 +8,12 @@ from xarray.coding import strings
 from xarray.core import indexing
 
 from . import (
-    IndexerMaker, assert_array_equal, assert_identical, raises_regex,
-    requires_dask)
+    IndexerMaker,
+    assert_array_equal,
+    assert_identical,
+    raises_regex,
+    requires_dask,
+)
 
 with suppress(ImportError):
     import dask.array as da
@@ -18,13 +21,13 @@ with suppress(ImportError):
 
 def test_vlen_dtype():
     dtype = strings.create_vlen_dtype(str)
-    assert dtype.metadata['element_type'] == str
+    assert dtype.metadata["element_type"] == str
     assert strings.is_unicode_dtype(dtype)
     assert not strings.is_bytes_dtype(dtype)
     assert strings.check_vlen_dtype(dtype) is str
 
     dtype = strings.create_vlen_dtype(bytes)
-    assert dtype.metadata['element_type'] == bytes
+    assert dtype.metadata["element_type"] == bytes
     assert not strings.is_unicode_dtype(dtype)
     assert strings.is_bytes_dtype(dtype)
     assert strings.check_vlen_dtype(dtype) is bytes
@@ -35,12 +38,11 @@ def test_vlen_dtype():
 def test_EncodedStringCoder_decode():
     coder = strings.EncodedStringCoder()
 
-    raw_data = np.array([b'abc', 'ß∂µ∆'.encode('utf-8')])
-    raw = Variable(('x',), raw_data, {'_Encoding': 'utf-8'})
+    raw_data = np.array([b"abc", "ß∂µ∆".encode()])
+    raw = Variable(("x",), raw_data, {"_Encoding": "utf-8"})
     actual = coder.decode(raw)
 
-    expected = Variable(
-        ('x',), np.array(['abc', 'ß∂µ∆'], dtype=object))
+    expected = Variable(("x",), np.array(["abc", "ß∂µ∆"], dtype=object))
     assert_identical(actual, expected)
 
     assert_identical(coder.decode(actual[0]), expected[0])
@@ -50,12 +52,12 @@ def test_EncodedStringCoder_decode():
 def test_EncodedStringCoder_decode_dask():
     coder = strings.EncodedStringCoder()
 
-    raw_data = np.array([b'abc', 'ß∂µ∆'.encode('utf-8')])
-    raw = Variable(('x',), raw_data, {'_Encoding': 'utf-8'}).chunk()
+    raw_data = np.array([b"abc", "ß∂µ∆".encode()])
+    raw = Variable(("x",), raw_data, {"_Encoding": "utf-8"}).chunk()
     actual = coder.decode(raw)
     assert isinstance(actual.data, da.Array)
 
-    expected = Variable(('x',), np.array(['abc', 'ß∂µ∆'], dtype=object))
+    expected = Variable(("x",), np.array(["abc", "ß∂µ∆"], dtype=object))
     assert_identical(actual, expected)
 
     actual_indexed = coder.decode(actual[0])
@@ -65,52 +67,72 @@ def test_EncodedStringCoder_decode_dask():
 
 def test_EncodedStringCoder_encode():
     dtype = strings.create_vlen_dtype(str)
-    raw_data = np.array(['abc', 'ß∂µ∆'], dtype=dtype)
-    expected_data = np.array([r.encode('utf-8') for r in raw_data],
-                             dtype=object)
+    raw_data = np.array(["abc", "ß∂µ∆"], dtype=dtype)
+    expected_data = np.array([r.encode("utf-8") for r in raw_data], dtype=object)
 
     coder = strings.EncodedStringCoder(allows_unicode=True)
-    raw = Variable(('x',), raw_data, encoding={'dtype': 'S1'})
+    raw = Variable(("x",), raw_data, encoding={"dtype": "S1"})
     actual = coder.encode(raw)
-    expected = Variable(('x',), expected_data, attrs={'_Encoding': 'utf-8'})
+    expected = Variable(("x",), expected_data, attrs={"_Encoding": "utf-8"})
     assert_identical(actual, expected)
 
-    raw = Variable(('x',), raw_data)
+    raw = Variable(("x",), raw_data)
     assert_identical(coder.encode(raw), raw)
 
     coder = strings.EncodedStringCoder(allows_unicode=False)
     assert_identical(coder.encode(raw), expected)
 
 
-@pytest.mark.parametrize('original', [
-    Variable(('x',), [b'ab', b'cdef']),
-    Variable((), b'ab'),
-    Variable(('x',), [b'a', b'b']),
-    Variable((), b'a'),
-])
+@pytest.mark.parametrize(
+    "original",
+    [
+        Variable(("x",), [b"ab", b"cdef"]),
+        Variable((), b"ab"),
+        Variable(("x",), [b"a", b"b"]),
+        Variable((), b"a"),
+    ],
+)
 def test_CharacterArrayCoder_roundtrip(original):
     coder = strings.CharacterArrayCoder()
     roundtripped = coder.decode(coder.encode(original))
     assert_identical(original, roundtripped)
 
 
-@pytest.mark.parametrize('data', [
-    np.array([b'a', b'bc']),
-    np.array([b'a', b'bc'], dtype=strings.create_vlen_dtype(bytes)),
-])
+@pytest.mark.parametrize(
+    "data",
+    [
+        np.array([b"a", b"bc"]),
+        np.array([b"a", b"bc"], dtype=strings.create_vlen_dtype(bytes)),
+    ],
+)
 def test_CharacterArrayCoder_encode(data):
     coder = strings.CharacterArrayCoder()
-    raw = Variable(('x',), data)
+    raw = Variable(("x",), data)
     actual = coder.encode(raw)
-    expected = Variable(('x', 'string2'),
-                        np.array([[b'a', b''], [b'b', b'c']]))
+    expected = Variable(("x", "string2"), np.array([[b"a", b""], [b"b", b"c"]]))
     assert_identical(actual, expected)
 
 
+@pytest.mark.parametrize(
+    ["original", "expected_char_dim_name"],
+    [
+        (Variable(("x",), [b"ab", b"cdef"]), "string4"),
+        (Variable(("x",), [b"ab", b"cdef"], encoding={"char_dim_name": "foo"}), "foo"),
+    ],
+)
+def test_CharacterArrayCoder_char_dim_name(original, expected_char_dim_name):
+    coder = strings.CharacterArrayCoder()
+    encoded = coder.encode(original)
+    roundtripped = coder.decode(encoded)
+    assert encoded.dims[-1] == expected_char_dim_name
+    assert roundtripped.encoding["char_dim_name"] == expected_char_dim_name
+    assert roundtripped.dims[-1] == original.dims[-1]
+
+
 def test_StackedBytesArray():
-    array = np.array([[b'a', b'b', b'c'], [b'd', b'e', b'f']], dtype='S')
+    array = np.array([[b"a", b"b", b"c"], [b"d", b"e", b"f"]], dtype="S")
     actual = strings.StackedBytesArray(array)
-    expected = np.array([b'abc', b'def'], dtype='S')
+    expected = np.array([b"abc", b"def"], dtype="S")
     assert actual.dtype == expected.dtype
     assert actual.shape == expected.shape
     assert actual.size == expected.size
@@ -125,10 +147,10 @@ def test_StackedBytesArray():
 
 
 def test_StackedBytesArray_scalar():
-    array = np.array([b'a', b'b', b'c'], dtype='S')
+    array = np.array([b"a", b"b", b"c"], dtype="S")
     actual = strings.StackedBytesArray(array)
 
-    expected = np.array(b'abc')
+    expected = np.array(b"abc")
     assert actual.dtype == expected.dtype
     assert actual.shape == expected.shape
     assert actual.size == expected.size
@@ -143,9 +165,9 @@ def test_StackedBytesArray_scalar():
 
 
 def test_StackedBytesArray_vectorized_indexing():
-    array = np.array([[b'a', b'b', b'c'], [b'd', b'e', b'f']], dtype='S')
+    array = np.array([[b"a", b"b", b"c"], [b"d", b"e", b"f"]], dtype="S")
     stacked = strings.StackedBytesArray(array)
-    expected = np.array([[b'abc', b'def'], [b'def', b'abc']])
+    expected = np.array([[b"abc", b"def"], [b"def", b"abc"]])
 
     V = IndexerMaker(indexing.VectorizedIndexer)
     indexer = V[np.array([[0, 1], [1, 0]])]
@@ -154,64 +176,62 @@ def test_StackedBytesArray_vectorized_indexing():
 
 
 def test_char_to_bytes():
-    array = np.array([[b'a', b'b', b'c'], [b'd', b'e', b'f']])
-    expected = np.array([b'abc', b'def'])
+    array = np.array([[b"a", b"b", b"c"], [b"d", b"e", b"f"]])
+    expected = np.array([b"abc", b"def"])
     actual = strings.char_to_bytes(array)
     assert_array_equal(actual, expected)
 
-    expected = np.array([b'ad', b'be', b'cf'])
+    expected = np.array([b"ad", b"be", b"cf"])
     actual = strings.char_to_bytes(array.T)  # non-contiguous
     assert_array_equal(actual, expected)
 
 
 def test_char_to_bytes_ndim_zero():
-    expected = np.array(b'a')
+    expected = np.array(b"a")
     actual = strings.char_to_bytes(expected)
     assert_array_equal(actual, expected)
 
 
 def test_char_to_bytes_size_zero():
-    array = np.zeros((3, 0), dtype='S1')
-    expected = np.array([b'', b'', b''])
+    array = np.zeros((3, 0), dtype="S1")
+    expected = np.array([b"", b"", b""])
     actual = strings.char_to_bytes(array)
     assert_array_equal(actual, expected)
 
 
 @requires_dask
 def test_char_to_bytes_dask():
-    numpy_array = np.array([[b'a', b'b', b'c'], [b'd', b'e', b'f']])
+    numpy_array = np.array([[b"a", b"b", b"c"], [b"d", b"e", b"f"]])
     array = da.from_array(numpy_array, ((2,), (3,)))
-    expected = np.array([b'abc', b'def'])
+    expected = np.array([b"abc", b"def"])
     actual = strings.char_to_bytes(array)
     assert isinstance(actual, da.Array)
     assert actual.chunks == ((2,),)
-    assert actual.dtype == 'S3'
+    assert actual.dtype == "S3"
     assert_array_equal(np.array(actual), expected)
 
-    with raises_regex(ValueError, 'stacked dask character array'):
+    with raises_regex(ValueError, "stacked dask character array"):
         strings.char_to_bytes(array.rechunk(1))
 
 
 def test_bytes_to_char():
-    array = np.array([[b'ab', b'cd'], [b'ef', b'gh']])
-    expected = np.array([[[b'a', b'b'], [b'c', b'd']],
-                         [[b'e', b'f'], [b'g', b'h']]])
+    array = np.array([[b"ab", b"cd"], [b"ef", b"gh"]])
+    expected = np.array([[[b"a", b"b"], [b"c", b"d"]], [[b"e", b"f"], [b"g", b"h"]]])
     actual = strings.bytes_to_char(array)
     assert_array_equal(actual, expected)
 
-    expected = np.array([[[b'a', b'b'], [b'e', b'f']],
-                         [[b'c', b'd'], [b'g', b'h']]])
+    expected = np.array([[[b"a", b"b"], [b"e", b"f"]], [[b"c", b"d"], [b"g", b"h"]]])
     actual = strings.bytes_to_char(array.T)  # non-contiguous
     assert_array_equal(actual, expected)
 
 
 @requires_dask
 def test_bytes_to_char_dask():
-    numpy_array = np.array([b'ab', b'cd'])
+    numpy_array = np.array([b"ab", b"cd"])
     array = da.from_array(numpy_array, ((1, 1),))
-    expected = np.array([[b'a', b'b'], [b'c', b'd']])
+    expected = np.array([[b"a", b"b"], [b"c", b"d"]])
     actual = strings.bytes_to_char(array)
     assert isinstance(actual, da.Array)
     assert actual.chunks == ((1, 1), ((2,)))
-    assert actual.dtype == 'S1'
+    assert actual.dtype == "S1"
     assert_array_equal(np.array(actual), expected)
