@@ -7,8 +7,8 @@ import xarray as xr
 from . import raises_regex
 
 
-@xr.register_dataset_accessor('example_accessor')
-@xr.register_dataarray_accessor('example_accessor')
+@xr.register_dataset_accessor("example_accessor")
+@xr.register_dataarray_accessor("example_accessor")
 class ExampleAccessor:
     """For the pickling tests below."""
 
@@ -18,9 +18,8 @@ class ExampleAccessor:
 
 class TestAccessor:
     def test_register(self):
-
-        @xr.register_dataset_accessor('demo')
-        @xr.register_dataarray_accessor('demo')
+        @xr.register_dataset_accessor("demo")
+        @xr.register_dataarray_accessor("demo")
         class DemoAccessor:
             """Demo accessor."""
 
@@ -29,13 +28,13 @@ class TestAccessor:
 
             @property
             def foo(self):
-                return 'bar'
+                return "bar"
 
         ds = xr.Dataset()
-        assert ds.demo.foo == 'bar'
+        assert ds.demo.foo == "bar"
 
         da = xr.DataArray(0)
-        assert da.demo.foo == 'bar'
+        assert da.demo.foo == "bar"
 
         # accessor is cached
         assert ds.demo is ds.demo
@@ -48,15 +47,16 @@ class TestAccessor:
 
         # ensure we can remove it
         del xr.Dataset.demo
-        assert not hasattr(xr.Dataset, 'demo')
+        assert not hasattr(xr.Dataset, "demo")
 
-        with pytest.warns(Warning, match='overriding a preexisting attribute'):
-            @xr.register_dataarray_accessor('demo')
+        with pytest.warns(Warning, match="overriding a preexisting attribute"):
+
+            @xr.register_dataarray_accessor("demo")
             class Foo:
                 pass
 
         # it didn't get registered again
-        assert not hasattr(xr.Dataset, 'demo')
+        assert not hasattr(xr.Dataset, "demo")
 
     def test_pickle_dataset(self):
         ds = xr.Dataset()
@@ -65,10 +65,10 @@ class TestAccessor:
 
         # state save on the accessor is restored
         assert ds.example_accessor is ds.example_accessor
-        ds.example_accessor.value = 'foo'
+        ds.example_accessor.value = "foo"
         ds_restored = pickle.loads(pickle.dumps(ds))
         assert ds.identical(ds_restored)
-        assert ds_restored.example_accessor.value == 'foo'
+        assert ds_restored.example_accessor.value == "foo"
 
     def test_pickle_dataarray(self):
         array = xr.Dataset()
@@ -79,10 +79,10 @@ class TestAccessor:
     def test_broken_accessor(self):
         # regression test for GH933
 
-        @xr.register_dataset_accessor('stupid_accessor')
+        @xr.register_dataset_accessor("stupid_accessor")
         class BrokenAccessor:
             def __init__(self, xarray_obj):
-                raise AttributeError('broken')
+                raise AttributeError("broken")
 
-        with raises_regex(RuntimeError, 'error initializing'):
+        with raises_regex(RuntimeError, "error initializing"):
             xr.Dataset().stupid_accessor
