@@ -1,5 +1,6 @@
+import contextlib
 import inspect
-from copy import deepcopy
+from copy import copy
 from datetime import datetime
 
 import numpy as np
@@ -44,6 +45,34 @@ except ImportError:
     ctpy = None
 
 
+@contextlib.contextmanager
+def figure_context(*args, **kwargs):
+    """context manager which autocloses a figure (even if the test failed)"""
+
+    try:
+        yield None
+    finally:
+        plt.close("all")
+
+
+@pytest.fixture(scope="module", autouse=True)
+def test_all_figures_closed():
+    """meta-test to ensure all figures are closed at the end of a test
+
+    Notes:  Scope is kept to module (only invoke this function once per test
+    module) else tests cannot be run in parallel (locally). Disadvantage: only
+    catches one open figure per run. May still give a false positive if tests
+    are run in parallel.
+    """
+    yield None
+
+    open_figs = len(plt.get_fignums())
+    if open_figs:
+        raise RuntimeError(
+            f"tests did not close all figures ({open_figs} figures open)"
+        )
+
+
 @pytest.mark.flaky
 @pytest.mark.skip(reason="maybe flaky")
 def text_in_fig():
@@ -284,12 +313,13 @@ class TestPlot(PlotTestCase):
             coords={"lat": (("y", "x"), lat), "lon": (("y", "x"), lon)},
         )
 
-        hdl = da.plot.line(x="lon", hue="x")
-        assert len(hdl) == 5
+        with figure_context():
+            hdl = da.plot.line(x="lon", hue="x")
+            assert len(hdl) == 5
 
-        plt.clf()
-        hdl = da.plot.line(x="lon", hue="y")
-        assert len(hdl) == 4
+        with figure_context():
+            hdl = da.plot.line(x="lon", hue="y")
+            assert len(hdl) == 4
 
         with pytest.raises(ValueError, match="For 2D inputs, hue must be a dimension"):
             da.plot.line(x="lon", hue="lat")
@@ -335,7 +365,7 @@ class TestPlot(PlotTestCase):
 
         cmap = mpl.cm.viridis
 
-        # deepcopy to ensure cmap is not changed by contourf()
+        # use copy to ensure cmap is not changed by contourf()
         # Set vmin and vmax so that _build_discrete_colormap is called with
         # extend='both'. extend is passed to
         # mpl.colors.from_levels_and_colors(), which returns a result with
@@ -343,12 +373,12 @@ class TestPlot(PlotTestCase):
         # extend='neither' (but if extend='neither' the under and over values
         # would not be used because the data would all be within the plotted
         # range)
-        pl = a.plot.contourf(cmap=deepcopy(cmap), vmin=0.1, vmax=0.9)
+        pl = a.plot.contourf(cmap=copy(cmap), vmin=0.1, vmax=0.9)
 
         # check the set_bad color
-        assert np.all(
-            pl.cmap(np.ma.masked_invalid([np.nan]))[0]
-            == cmap(np.ma.masked_invalid([np.nan]))[0]
+        assert_array_equal(
+            pl.cmap(np.ma.masked_invalid([np.nan]))[0],
+            cmap(np.ma.masked_invalid([np.nan]))[0],
         )
 
         # check the set_under color
@@ -360,10 +390,8 @@ class TestPlot(PlotTestCase):
     def test_contourf_cmap_set_with_bad_under_over(self):
         a = DataArray(easy_array((4, 4)), dims=["z", "time"])
 
-        # Make a copy here because we want a local cmap that we will modify.
-        # Use deepcopy because matplotlib Colormap objects have tuple members
-        # and we want to ensure we do not change the original.
-        cmap = deepcopy(mpl.cm.viridis)
+        # make a copy here because we want a local cmap that we will modify.
+        cmap = copy(mpl.cm.viridis)
 
         cmap.set_bad("w")
         # check we actually changed the set_bad color
@@ -380,13 +408,13 @@ class TestPlot(PlotTestCase):
         # check we actually changed the set_over color
         assert cmap(np.inf) != mpl.cm.viridis(-np.inf)
 
-        # deepcopy to ensure cmap is not changed by contourf()
-        pl = a.plot.contourf(cmap=deepcopy(cmap))
+        # copy to ensure cmap is not changed by contourf()
+        pl = a.plot.contourf(cmap=copy(cmap))
 
         # check the set_bad color has been kept
-        assert np.all(
-            pl.cmap(np.ma.masked_invalid([np.nan]))[0]
-            == cmap(np.ma.masked_invalid([np.nan]))[0]
+        assert_array_equal(
+            pl.cmap(np.ma.masked_invalid([np.nan]))[0],
+            cmap(np.ma.masked_invalid([np.nan]))[0],
         )
 
         # check the set_under color has been kept
@@ -643,11 +671,13 @@ class TestPlotStep(PlotTestCase):
         self.darray = DataArray(easy_array((2, 3, 4)))
 
     def test_step(self):
-        self.darray[0, 0].plot.step()
+        hdl = self.darray[0, 0].plot.step()
+        assert "steps" in hdl[0].get_drawstyle()
 
-    @pytest.mark.parametrize("ds", ["pre", "post", "mid"])
-    def test_step_with_drawstyle(self, ds):
-        self.darray[0, 0].plot.step(drawstyle=ds)
+    @pytest.mark.parametrize("where", ["pre", "post", "mid"])
+    def test_step_with_where(self, where):
+        hdl = self.darray[0, 0].plot.step(where=where)
+        assert hdl[0].get_drawstyle() == f"steps-{where}"
 
     def test_coord_with_interval_step(self):
         """Test step plot with intervals."""
@@ -765,18 +795,22 @@ class TestDetermineCmapParams:
         # default is to cover full data range but with no guarantee on Nlevels
         for level in np.arange(2, 10, dtype=int):
             cmap_params = _determine_cmap_params(data, levels=level)
-            assert cmap_params["vmin"] == cmap_params["levels"][0]
-            assert cmap_params["vmax"] == cmap_params["levels"][-1]
+            assert cmap_params["vmin"] is None
+            assert cmap_params["vmax"] is None
+            assert cmap_params["norm"].vmin == cmap_params["levels"][0]
+            assert cmap_params["norm"].vmax == cmap_params["levels"][-1]
             assert cmap_params["extend"] == "neither"
 
         # with min max we are more strict
         cmap_params = _determine_cmap_params(
             data, levels=5, vmin=0, vmax=5, cmap="Blues"
         )
-        assert cmap_params["vmin"] == 0
-        assert cmap_params["vmax"] == 5
-        assert cmap_params["vmin"] == cmap_params["levels"][0]
-        assert cmap_params["vmax"] == cmap_params["levels"][-1]
+        assert cmap_params["vmin"] is None
+        assert cmap_params["vmax"] is None
+        assert cmap_params["norm"].vmin == 0
+        assert cmap_params["norm"].vmax == 5
+        assert cmap_params["norm"].vmin == cmap_params["levels"][0]
+        assert cmap_params["norm"].vmax == cmap_params["levels"][-1]
         assert cmap_params["cmap"].name == "Blues"
         assert cmap_params["extend"] == "neither"
         assert cmap_params["cmap"].N == 4
@@ -800,8 +834,10 @@ class TestDetermineCmapParams:
         orig_levels = [0, 1, 2, 3, 4, 5]
         # vmin and vmax should be ignored if levels are explicitly provided
         cmap_params = _determine_cmap_params(data, levels=orig_levels, vmin=0, vmax=3)
-        assert cmap_params["vmin"] == 0
-        assert cmap_params["vmax"] == 5
+        assert cmap_params["vmin"] is None
+        assert cmap_params["vmax"] is None
+        assert cmap_params["norm"].vmin == 0
+        assert cmap_params["norm"].vmax == 5
         assert cmap_params["cmap"].N == 5
         assert cmap_params["norm"].N == 6
 
@@ -907,8 +943,10 @@ class TestDetermineCmapParams:
             test_max = vmax if norm.vmax is None else norm.vmax
 
             cmap_params = _determine_cmap_params(self.data, norm=norm, levels=levels)
-            assert cmap_params["vmin"] == test_min
-            assert cmap_params["vmax"] == test_max
+            assert cmap_params["vmin"] is None
+            assert cmap_params["vmax"] is None
+            assert cmap_params["norm"].vmin == test_min
+            assert cmap_params["norm"].vmax == test_max
             assert cmap_params["extend"] == extend
             assert cmap_params["norm"] == norm
 
@@ -924,6 +962,9 @@ class TestDiscreteColorMap:
         self.darray = DataArray(distance, list(zip(("y", "x"), (y, x))))
         self.data_min = distance.min()
         self.data_max = distance.max()
+        yield
+        # Remove all matplotlib figures
+        plt.close("all")
 
     @pytest.mark.slow
     def test_recover_from_seaborn_jet_exception(self):
@@ -1719,14 +1760,15 @@ class TestImshow(Common2dMixin, PlotTestCase):
 
     def test_origin_overrides_xyincrease(self):
         da = DataArray(easy_array((3, 2)), coords=[[-2, 0, 2], [-1, 1]])
-        da.plot.imshow(origin="upper")
-        assert plt.xlim()[0] < 0
-        assert plt.ylim()[1] < 0
+        with figure_context():
+            da.plot.imshow(origin="upper")
+            assert plt.xlim()[0] < 0
+            assert plt.ylim()[1] < 0
 
-        plt.clf()
-        da.plot.imshow(origin="lower")
-        assert plt.xlim()[0] < 0
-        assert plt.ylim()[0] < 0
+        with figure_context():
+            da.plot.imshow(origin="lower")
+            assert plt.xlim()[0] < 0
+            assert plt.ylim()[0] < 0
 
 
 class TestFacetGrid(PlotTestCase):
@@ -2227,6 +2269,13 @@ class TestDatasetScatterPlots(PlotTestCase):
         with pytest.raises(ValueError):
             ds2.plot.scatter(x="A", y="B", hue="hue", hue_style="continuous")
 
+    def test_legend_labels(self):
+        # regression test for #4126: incorrect legend labels
+        ds2 = self.ds.copy()
+        ds2["hue"] = ["a", "a", "b", "b"]
+        lines = ds2.plot.scatter(x="A", y="B", hue="hue")
+        assert [t.get_text() for t in lines[0].axes.get_legend().texts] == ["a", "b"]
+
     def test_add_legend_by_default(self):
         sc = self.ds.plot.scatter(x="A", y="B", hue="hue")
         assert len(sc.figure.axes) == 2
@@ -2251,6 +2300,7 @@ class TestDatetimePlot(PlotTestCase):
         self.darray.plot.line()
 
 
+@pytest.mark.filterwarnings("ignore:setting an array element with a sequence")
 @requires_nc_time_axis
 @requires_cftime
 class TestCFDatetimePlot(PlotTestCase):
@@ -2313,60 +2363,60 @@ class TestAxesKwargs:
     @pytest.mark.parametrize("da", test_da_list)
     @pytest.mark.parametrize("xincrease", [True, False])
     def test_xincrease_kwarg(self, da, xincrease):
-        plt.clf()
-        da.plot(xincrease=xincrease)
-        assert plt.gca().xaxis_inverted() == (not xincrease)
+        with figure_context():
+            da.plot(xincrease=xincrease)
+            assert plt.gca().xaxis_inverted() == (not xincrease)
 
     @pytest.mark.parametrize("da", test_da_list)
     @pytest.mark.parametrize("yincrease", [True, False])
     def test_yincrease_kwarg(self, da, yincrease):
-        plt.clf()
-        da.plot(yincrease=yincrease)
-        assert plt.gca().yaxis_inverted() == (not yincrease)
+        with figure_context():
+            da.plot(yincrease=yincrease)
+            assert plt.gca().yaxis_inverted() == (not yincrease)
 
     @pytest.mark.parametrize("da", test_da_list)
     @pytest.mark.parametrize("xscale", ["linear", "log", "logit", "symlog"])
     def test_xscale_kwarg(self, da, xscale):
-        plt.clf()
-        da.plot(xscale=xscale)
-        assert plt.gca().get_xscale() == xscale
+        with figure_context():
+            da.plot(xscale=xscale)
+            assert plt.gca().get_xscale() == xscale
 
     @pytest.mark.parametrize(
         "da", [DataArray(easy_array((10,))), DataArray(easy_array((10, 3)))]
     )
     @pytest.mark.parametrize("yscale", ["linear", "log", "logit", "symlog"])
     def test_yscale_kwarg(self, da, yscale):
-        plt.clf()
-        da.plot(yscale=yscale)
-        assert plt.gca().get_yscale() == yscale
+        with figure_context():
+            da.plot(yscale=yscale)
+            assert plt.gca().get_yscale() == yscale
 
     @pytest.mark.parametrize("da", test_da_list)
     def test_xlim_kwarg(self, da):
-        plt.clf()
-        expected = (0.0, 1000.0)
-        da.plot(xlim=[0, 1000])
-        assert plt.gca().get_xlim() == expected
+        with figure_context():
+            expected = (0.0, 1000.0)
+            da.plot(xlim=[0, 1000])
+            assert plt.gca().get_xlim() == expected
 
     @pytest.mark.parametrize("da", test_da_list)
     def test_ylim_kwarg(self, da):
-        plt.clf()
-        da.plot(ylim=[0, 1000])
-        expected = (0.0, 1000.0)
-        assert plt.gca().get_ylim() == expected
+        with figure_context():
+            da.plot(ylim=[0, 1000])
+            expected = (0.0, 1000.0)
+            assert plt.gca().get_ylim() == expected
 
     @pytest.mark.parametrize("da", test_da_list)
     def test_xticks_kwarg(self, da):
-        plt.clf()
-        da.plot(xticks=np.arange(5))
-        expected = np.arange(5).tolist()
-        assert np.all(plt.gca().get_xticks() == expected)
+        with figure_context():
+            da.plot(xticks=np.arange(5))
+            expected = np.arange(5).tolist()
+            assert_array_equal(plt.gca().get_xticks(), expected)
 
     @pytest.mark.parametrize("da", test_da_list)
     def test_yticks_kwarg(self, da):
-        plt.clf()
-        da.plot(yticks=np.arange(5))
-        expected = np.arange(5)
-        assert np.all(plt.gca().get_yticks() == expected)
+        with figure_context():
+            da.plot(yticks=np.arange(5))
+            expected = np.arange(5)
+            assert_array_equal(plt.gca().get_yticks(), expected)
 
 
 @requires_matplotlib
@@ -2381,8 +2431,10 @@ def test_plot_transposed_nondim_coord(plotfunc):
         dims=["s", "x"],
         coords={"x": x, "s": s, "z": (("s", "x"), z), "zt": (("x", "s"), z.T)},
     )
-    getattr(da.plot, plotfunc)(x="x", y="zt")
-    getattr(da.plot, plotfunc)(x="zt", y="x")
+    with figure_context():
+        getattr(da.plot, plotfunc)(x="x", y="zt")
+    with figure_context():
+        getattr(da.plot, plotfunc)(x="zt", y="x")
 
 
 @requires_matplotlib
@@ -2390,11 +2442,12 @@ def test_plot_transposed_nondim_coord(plotfunc):
 def test_plot_transposes_properly(plotfunc):
     # test that we aren't mistakenly transposing when the 2 dimensions have equal sizes.
     da = xr.DataArray([np.sin(2 * np.pi / 10 * np.arange(10))] * 10, dims=("y", "x"))
-    hdl = getattr(da.plot, plotfunc)(x="x", y="y")
-    # get_array doesn't work for contour, contourf. It returns the colormap intervals.
-    # pcolormesh returns 1D array but imshow returns a 2D array so it is necessary
-    # to ravel() on the LHS
-    assert np.all(hdl.get_array().ravel() == da.to_masked_array().ravel())
+    with figure_context():
+        hdl = getattr(da.plot, plotfunc)(x="x", y="y")
+        # get_array doesn't work for contour, contourf. It returns the colormap intervals.
+        # pcolormesh returns 1D array but imshow returns a 2D array so it is necessary
+        # to ravel() on the LHS
+        assert_array_equal(hdl.get_array().ravel(), da.to_masked_array().ravel())
 
 
 @requires_matplotlib
@@ -2406,7 +2459,8 @@ def test_facetgrid_single_contour():
     ds = xr.concat([z, z2], dim="time")
     ds["time"] = [0, 1]
 
-    ds.plot.contour(col="time", levels=[4], colors=["k"])
+    with figure_context():
+        ds.plot.contour(col="time", levels=[4], colors=["k"])
 
 
 @requires_matplotlib
@@ -2430,13 +2484,15 @@ def test_get_axis():
     with pytest.raises(ValueError, match="`aspect` argument without `size`"):
         get_axis(figsize=None, size=None, aspect=4 / 3, ax=None)
 
-    ax = get_axis()
-    assert isinstance(ax, mpl.axes.Axes)
+    with figure_context():
+        ax = get_axis()
+        assert isinstance(ax, mpl.axes.Axes)
 
 
 @requires_cartopy
 def test_get_axis_cartopy():
 
     kwargs = {"projection": ctpy.crs.PlateCarree()}
-    ax = get_axis(**kwargs)
-    assert isinstance(ax, ctpy.mpl.geoaxes.GeoAxesSubplot)
+    with figure_context():
+        ax = get_axis(**kwargs)
+        assert isinstance(ax, ctpy.mpl.geoaxes.GeoAxesSubplot)
